@@ -2,7 +2,9 @@
 #define _RAR_CONSIO_
 
 void InitConsole();
-void InitConsoleOptions(MESSAGE_TYPE MsgStream,RAR_CHARSET RedirectCharset);
+void SetConsoleMsgStream(MESSAGE_TYPE MsgStream);
+void SetConsoleRedirectCharset(RAR_CHARSET RedirectCharset);
+void ProhibitConsoleInput();
 void OutComment(const wchar *Comment,size_t Size);
 
 #ifndef SILENT
