@@ -28,26 +28,38 @@ class BitInput
       InAddr+=Bits>>3;
       InBit=Bits&7;
     }
-    
+
     // Return 16 bits from current position in the buffer.
     // Bit at (InAddr,InBit) has the highest position in returning data.
     uint getbits()
     {
+#if defined(LITTLE_ENDIAN) && defined(ALLOW_MISALIGNED)
+      uint32 BitField=*(uint32*)(InBuf+InAddr);
+      BitField=ByteSwap32(BitField);
+      BitField >>= (16-InBit);
+#else
       uint BitField=(uint)InBuf[InAddr] << 16;
       BitField|=(uint)InBuf[InAddr+1] << 8;
       BitField|=(uint)InBuf[InAddr+2];
       BitField >>= (8-InBit);
+#endif
       return BitField & 0xffff;
     }
 
+
     // Return 32 bits from current position in the buffer.
     // Bit at (InAddr,InBit) has the highest position in returning data.
     uint getbits32()
     {
+#if defined(LITTLE_ENDIAN) && defined(ALLOW_MISALIGNED)
+      uint32 BitField=*(uint32*)(InBuf+InAddr);
+      BitField=ByteSwap32(BitField);
+#else
       uint BitField=(uint)InBuf[InAddr] << 24;
       BitField|=(uint)InBuf[InAddr+1] << 16;
       BitField|=(uint)InBuf[InAddr+2] << 8;
       BitField|=(uint)InBuf[InAddr+3];
+#endif
       BitField <<= InBit;
       BitField|=(uint)InBuf[InAddr+4] >> (8-InBit);
       return BitField & 0xffffffff;
