@@ -5,7 +5,7 @@ wchar* PointToName(const wchar *Path)
   for (int I=(int)wcslen(Path)-1;I>=0;I--)
     if (IsPathDiv(Path[I]))
       return (wchar*)&Path[I+1];
-  return (wchar*)((*Path && IsDriveDiv(Path[1])) ? Path+2:Path);
+  return (wchar*)((*Path!=0 && IsDriveDiv(Path[1])) ? Path+2:Path);
 }
 
 
@@ -16,7 +16,7 @@ wchar* PointToLastChar(const wchar *Path)
 }
 
 
-wchar* ConvertPath(const wchar *SrcPath,wchar *DestPath)
+wchar* ConvertPath(const wchar *SrcPath,wchar *DestPath,size_t DestSize)
 {
   const wchar *DestPtr=SrcPath;
 
@@ -25,17 +25,23 @@ wchar* ConvertPath(const wchar *SrcPath,wchar *DestPath)
     if (IsPathDiv(s[0]) && s[1]=='.' && s[2]=='.' && IsPathDiv(s[3]))
       DestPtr=s+4;
 
-  // Remove <d>:\ and any sequence of . and \ in the beginning of path string.
+  // Remove any amount of <d>:\ and any sequence of . and \ in the beginning of path string.
   while (*DestPtr!=0)
   {
     const wchar *s=DestPtr;
     if (s[0]!=0 && IsDriveDiv(s[1]))
       s+=2;
-    if (s[0]=='\\' && s[1]=='\\')
+
+    // Skip UNC Windows \\server\share\ or Unix //server/share/
+    if (IsPathDiv(s[0]) && IsPathDiv(s[1]))
     {
-      const wchar *Slash=wcschr(s+2,'\\');
-      if (Slash!=NULL && (Slash=wcschr(Slash+1,'\\'))!=NULL)
-        s=Slash+1;
+      uint SlashCount=0;
+      for (const wchar *t=s+2;*t!=0;t++)
+        if (IsPathDiv(*t) && ++SlashCount==2)
+        {
+          s=t+1; // Found two more path separators after leading two.
+          break;
+        }
     }
     for (const wchar *t=s;*t!=0;t++)
       if (IsPathDiv(*t))
@@ -58,7 +64,7 @@ wchar* ConvertPath(const wchar *SrcPath,wchar *DestPath)
     // so we use the temporary buffer for copying.
     wchar TmpStr[NM];
     wcsncpyz(TmpStr,DestPtr,ASIZE(TmpStr));
-    wcscpy(DestPath,TmpStr);
+    wcsncpyz(DestPath,TmpStr,DestSize);
   }
   return (wchar *)DestPtr;
 }
@@ -120,7 +126,14 @@ bool CmpExt(const wchar *Name,const wchar *Ext)
 
 bool IsWildcard(const wchar *Str)
 {
-  return Str==NULL ? false:wcspbrk(Str,L"*?")!=NULL;
+  if (Str==NULL)
+    return false;
+#ifdef _WIN_ALL
+  // Not treat the special NTFS \\?\d: path prefix as a wildcard.
+  if (Str[0]=='\\' && Str[1]=='\\' && Str[2]=='?' && Str[3]=='\\')
+    Str+=4;
+#endif
+  return wcspbrk(Str,L"*?")!=NULL;
 }
 
 
@@ -164,17 +177,22 @@ void AddEndSlash(wchar *Path,size_t MaxLength)
 {
   size_t Length=wcslen(Path);
   if (Length>0 && Path[Length-1]!=CPATHDIVIDER && Length+1<MaxLength)
-    wcscat(Path,SPATHDIVIDER);
+  {
+    Path[Length]=CPATHDIVIDER;
+    Path[Length+1]=0;
+  }
 }
 
 
 void MakeName(const wchar *Path,const wchar *Name,wchar *Pathname,size_t MaxSize)
 {
-  // 'Name' and 'Pathname' can point to same memory area. This is why we use
+  // 'Path', 'Name' and 'Pathname' can point to same memory area. So we use
   // the temporary buffer instead of constructing the name in 'Pathname'.
   wchar OutName[NM];
   wcsncpyz(OutName,Path,ASIZE(OutName));
-  AddEndSlash(OutName,ASIZE(OutName));
+  // Do not add slash to d:, we want to allow relative paths like d:filename.
+  if (!IsDriveLetter(Path) || Path[2]!=0)
+    AddEndSlash(OutName,ASIZE(OutName));
   wcsncatz(OutName,Name,ASIZE(OutName));
   wcsncpyz(Pathname,OutName,MaxSize);
 }
@@ -260,9 +278,9 @@ bool EnumConfigPaths(uint Number,wchar *Path,size_t MaxSize,bool Create)
   {
     char *EnvStr=getenv("HOME");
     if (EnvStr!=NULL)
-      GetWideName(EnvStr,NULL,Path,MaxSize);
+      CharToWide(EnvStr,Path,MaxSize);
     else
-      wcsncpyz(Path, ConfPath[0], MaxSize);
+      wcsncpyz(Path,ConfPath[0],MaxSize);
     return true;
   }
   Number--;
@@ -303,9 +321,16 @@ void GetConfigName(const wchar *Name,wchar *FullName,size_t MaxSize,bool CheckEx
 #endif
 
 
-// Returns a pointer to rightmost digit of volume number.
+// Returns a pointer to rightmost digit of volume number or to beginning
+// of file name if numeric part is missing.
 wchar* GetVolNumPart(const wchar *ArcName)
 {
+  // We do not want to increment any characters in path component.
+  ArcName=PointToName(ArcName);
+
+  if (*ArcName==0)
+    return (wchar *)ArcName;
+
   // Pointing to last name character.
   const wchar *ChPtr=ArcName+wcslen(ArcName)-1;
 
@@ -326,7 +351,7 @@ wchar* GetVolNumPart(const wchar *ArcName)
     {
       // Validate the first numeric part only if it has a dot somewhere 
       // before it.
-      wchar *Dot=wcschr(PointToName(ArcName),'.');
+      const wchar *Dot=wcschr(ArcName,'.');
       if (Dot!=NULL && Dot<NumPtr)
         ChPtr=NumPtr;
       break;
@@ -346,18 +371,33 @@ void NextVolumeName(wchar *ArcName,uint MaxLength,bool OldNumbering)
     ChPtr=GetExt(ArcName);
   }
   else
-    if (ChPtr[1]==0 && wcslen(ArcName)<MaxLength-3 || wcsicomp(ChPtr+1,L"exe")==0 || wcsicomp(ChPtr+1,L"sfx")==0)
-      wcscpy(ChPtr+1,L"rar");
+    if (ChPtr[1]==0 || wcsicomp(ChPtr,L".exe")==0 || wcsicomp(ChPtr,L".sfx")==0)
+      wcsncpyz(ChPtr,L".rar",MaxLength-(ChPtr-ArcName));
+
+  if (ChPtr==NULL || *ChPtr!='.' || ChPtr[1]==0)
+  {
+    // Normally we shall have some extension here. If we don't, it means
+    // the name has no extension and buffer has no free space to append one.
+    // Let's clear the name to prevent a new call with same name and return.
+    *ArcName=0;
+    return;
+  }
+
   if (!OldNumbering)
   {
     ChPtr=GetVolNumPart(ArcName);
 
+    // We should not check for IsDigit(*ChPtr) here and should increment
+    // even non-digits. If we got a corrupt archive with volume flag,
+    // but without numeric part, we still need to modify its name somehow,
+    // so while (exist(name)) {NextVolumeName()} loops do not run infinitely.
     while ((++(*ChPtr))=='9'+1)
     {
       *ChPtr='0';
       ChPtr--;
       if (ChPtr<ArcName || !IsDigit(*ChPtr))
       {
+        // Convert .part:.rar (.part9.rar after increment) to part10.rar.
         for (wchar *EndPtr=ArcName+wcslen(ArcName);EndPtr!=ChPtr;EndPtr--)
           *(EndPtr+1)=*EndPtr;
         *(ChPtr+1)='1';
@@ -366,15 +406,15 @@ void NextVolumeName(wchar *ArcName,uint MaxLength,bool OldNumbering)
     }
   }
   else
-    if (!IsDigit(*(ChPtr+2)) || !IsDigit(*(ChPtr+3)))
-      wcscpy(ChPtr+2,L"00");
+    if (!IsDigit(ChPtr[2]) || !IsDigit(ChPtr[3]))
+      wcsncpyz(ChPtr+2,L"00",MaxLength-(ChPtr-ArcName)-2); // From .rar to .r00.
     else
     {
-      ChPtr+=3;
-      while ((++(*ChPtr))=='9'+1)
-        if (*(ChPtr-1)=='.')
+      ChPtr+=wcslen(ChPtr)-1; // Set to last character.
+      while (++(*ChPtr)=='9'+1)
+        if (ChPtr<=ArcName || *(ChPtr-1)=='.')
         {
-          *ChPtr='A';
+          *ChPtr='a'; // From .999 to .a00 if started from .001 or for too short names.
           break;
         }
         else
@@ -388,50 +428,39 @@ void NextVolumeName(wchar *ArcName,uint MaxLength,bool OldNumbering)
 
 bool IsNameUsable(const wchar *Name)
 {
-#ifndef _UNIX
-  if (Name[0] && Name[1] && wcschr(Name+2,':')!=NULL)
+  // We were asked to apply Windows-like conversion in Linux in case
+  // files are unpacked to Windows share. This code is invoked only
+  // if file failed to be created, so it doesn't affect extraction
+  // of Unix compatible names to native Unix drives.
+#ifdef _UNIX
+  // Windows shares in Unix do not allow the drive letter,
+  // so unlike Windows version, we check all characters here.
+  if (wcschr(Name,':')!=NULL)
     return false;
+#else
+  if (Name[0]!=0 && Name[1]!=0 && wcschr(Name+2,':')!=NULL)
+    return false;
+#endif
   for (const wchar *s=Name;*s!=0;s++)
   {
     if ((uint)*s<32)
       return false;
+
+     // It is for Windows shares in Unix. We can create such names in Windows.
+#ifdef _UNIX
+    // No spaces or dots before the path separator are allowed in Windows
+    // shares. But they are allowed and automtically removed at the end of
+    // file or folder name, so it is useless to replace them here.
+    // Since such files or folders are created successfully, a supposed
+    // conversion here would never be invoked.
     if ((*s==' ' || *s=='.') && IsPathDiv(s[1]))
       return false;
-  }
 #endif
+  }
   return *Name!=0 && wcspbrk(Name,L"?*<>|\"")==NULL;
 }
 
 
-void MakeNameUsable(char *Name,bool Extended)
-{
-#ifdef _WIN_ALL
-  // In Windows we also need to convert characters not defined in current
-  // code page. This double conversion changes them to '?', which is
-  // catched by code below.
-  size_t NameLength=strlen(Name);
-  wchar NameW[NM];
-  CharToWide(Name,NameW,ASIZE(NameW));
-  WideToChar(NameW,Name,NameLength+1);
-  Name[NameLength]=0;
-#endif
-  for (char *s=Name;*s!=0;s=charnext(s))
-  {
-    if (strchr(Extended ? "?*<>|\"":"?*",*s)!=NULL || Extended && (byte)*s<32)
-      *s='_';
-#ifdef _EMX
-    if (*s=='=')
-      *s='_';
-#endif
-#ifndef _UNIX
-    if (s-Name>1 && *s==':')
-      *s='_';
-    // Remove ' ' and '.' before path separator, but allow .\ and ..\.
-    if ((*s==' ' || *s=='.' && s>Name && !IsPathDiv(s[-1]) && s[-1]!='.') && IsPathDiv(s[1]))
-      *s='_';
-#endif
-  }
-}
 
 
 void MakeNameUsable(wchar *Name,bool Extended)
@@ -440,7 +469,27 @@ void MakeNameUsable(wchar *Name,bool Extended)
   {
     if (wcschr(Extended ? L"?*<>|\"":L"?*",*s)!=NULL || Extended && (uint)*s<32)
       *s='_';
-#ifndef _UNIX
+#ifdef _UNIX
+    // We were asked to apply Windows-like conversion in Linux in case
+    // files are unpacked to Windows share. This code is invoked only
+    // if file failed to be created, so it doesn't affect extraction
+    // of Unix compatible names to native Unix drives.
+    if (Extended)
+    {
+      // Windows shares in Unix do not allow the drive letter,
+      // so unlike Windows version, we check all characters here.
+      if (*s==':')
+        *s='_';
+
+      // No spaces or dots before the path separator are allowed on Windows
+      // shares. But they are allowed and automtically removed at the end of
+      // file or folder name, so it is useless to replace them here.
+      // Since such files or folders are created successfully, a supposed
+      // conversion here would never be invoked.
+      if ((*s==' ' || *s=='.') && IsPathDiv(s[1]))
+        *s='_';
+    }
+#else
     if (s-Name>1 && *s==':')
       *s='_';
 #if 0  // We already can create such files.
@@ -585,8 +634,7 @@ int ParseVersionFileName(wchar *Name,bool Truncate)
   wchar *VerText=wcsrchr(Name,';');
   if (VerText!=NULL)
   {
-    if (Version==0)
-      Version=atoiw(VerText+1);
+    Version=atoiw(VerText+1);
     if (Truncate)
       *VerText=0;
   }
@@ -627,7 +675,7 @@ wchar* VolNameToFirstName(const wchar *VolName,wchar *FirstName,size_t MaxSize,b
   }
   if (!FileExist(FirstName))
   {
-    // If the first volume, which name we just generated, is not exist,
+    // If the first volume, which name we just generated, does not exist,
     // check if volume with same name and any other extension is available.
     // It can help in case of *.exe or *.sfx first volume.
     wchar Mask[NM];
@@ -652,7 +700,7 @@ wchar* VolNameToFirstName(const wchar *VolName,wchar *FirstName,size_t MaxSize,b
 
 
 #ifndef SFX_MODULE
-static void GenArcName(wchar *ArcName,const wchar *GenerateMask,uint ArcNumber,bool &ArcNumPresent)
+static void GenArcName(wchar *ArcName,size_t MaxSize,const wchar *GenerateMask,uint ArcNumber,bool &ArcNumPresent)
 {
   bool Prefix=false;
   if (*GenerateMask=='+')
@@ -664,7 +712,8 @@ static void GenArcName(wchar *ArcName,const wchar *GenerateMask,uint ArcNumber,b
   wchar Mask[MAX_GENERATE_MASK];
   wcsncpyz(Mask,*GenerateMask!=0 ? GenerateMask:L"yyyymmddhhmmss",ASIZE(Mask));
 
-  bool QuoteMode=false,Hours=false;
+  bool QuoteMode=false;
+  uint MAsMinutes=0; // By default we treat 'M' as months.
   for (uint I=0;Mask[I]!=0;I++)
   {
     if (Mask[I]=='{' || Mask[I]=='}')
@@ -676,13 +725,16 @@ static void GenArcName(wchar *ArcName,const wchar *GenerateMask,uint ArcNumber,b
       continue;
     int CurChar=toupperw(Mask[I]);
     if (CurChar=='H')
-      Hours=true;
+      MAsMinutes=2; // Treat next two 'M' after 'H' as minutes.
+    if (CurChar=='D' || CurChar=='Y')
+      MAsMinutes=0; // Treat 'M' in HHDDMMYY and HHYYMMDD as month.
 
-    if (Hours && CurChar=='M')
+    if (MAsMinutes>0 && CurChar=='M')
     {
       // Replace minutes with 'I'. We use 'M' both for months and minutes,
-      // so we treat as minutes only those 'M' which are found after hours.
+      // so we treat as minutes only those 'M', which are found after hours.
       Mask[I]='I';
+      MAsMinutes--;
     }
     if (CurChar=='N')
     {
@@ -713,7 +765,7 @@ static void GenArcName(wchar *ArcName,const wchar *GenerateMask,uint ArcNumber,b
   wchar Ext[NM],*Dot=GetExt(ArcName);
   *Ext=0;
   if (Dot==NULL)
-    wcscpy(Ext,*PointToName(ArcName)==0 ? L".rar":L"");
+    wcsncpyz(Ext,*PointToName(ArcName)==0 ? L".rar":L"",ASIZE(Ext));
   else
   {
     wcsncpyz(Ext,Dot,ASIZE(Ext));
@@ -746,10 +798,12 @@ static void GenArcName(wchar *ArcName,const wchar *GenerateMask,uint ArcNumber,b
 
   const wchar *MaskChars=L"YMDHISWAEN";
 
+  // How many times every modifier character was encountered in the mask.
   int CField[sizeof(Field)/sizeof(Field[0])];
+
   memset(CField,0,sizeof(CField));
   QuoteMode=false;
-  for (int I=0;Mask[I]!=0;I++)
+  for (uint I=0;Mask[I]!=0;I++)
   {
     if (Mask[I]=='{' || Mask[I]=='}')
     {
@@ -788,13 +842,22 @@ static void GenArcName(wchar *ArcName,const wchar *GenerateMask,uint ArcNumber,b
     {
       size_t FieldPos=ChPtr-MaskChars;
       int CharPos=(int)strlen(Field[FieldPos])-CField[FieldPos]--;
-      if (FieldPos==1 && toupperw(Mask[I+1])=='M' && toupperw(Mask[I+2])=='M')
+
+      // CField[FieldPos] shall have exactly 3 "MMM" symbols, so we do not
+      // repeat the month name in case "MMMMMMMM" mask. But since we
+      // decremented CField[FieldPos] above, we compared it with 2.
+      if (FieldPos==1 && CField[FieldPos]==2 &&
+          toupperw(Mask[I+1])=='M' && toupperw(Mask[I+2])=='M')
       {
         wcsncpyz(DateText+J,GetMonthName(rlt.Month-1),ASIZE(DateText)-J);
         J=wcslen(DateText);
         I+=2;
         continue;
       }
+      // If CharPos is negative, we have more modifier characters than
+      // matching time data. We prefer to issue a modifier character
+      // instead of repeating time data from beginning, so user can notice
+      // excessive modifiers added by mistake.
       if (CharPos<0)
         DateText[J]=Mask[I];
       else
@@ -810,21 +873,17 @@ static void GenArcName(wchar *ArcName,const wchar *GenerateMask,uint ArcNumber,b
     AddEndSlash(NewName,ASIZE(NewName));
     wcsncatz(NewName,DateText,ASIZE(NewName));
     wcsncatz(NewName,PointToName(ArcName),ASIZE(NewName));
-    wcscpy(ArcName,NewName);
+    wcsncpyz(ArcName,NewName,MaxSize);
   }
   else
-    wcscat(ArcName,DateText);
-  wcscat(ArcName,Ext);
+    wcsncatz(ArcName,DateText,MaxSize);
+  wcsncatz(ArcName,Ext,MaxSize);
 }
 
 
 void GenerateArchiveName(wchar *ArcName,size_t MaxSize,const wchar *GenerateMask,bool Archiving)
 {
-  // Must be enough space for archive name plus all stuff in mask plus
-  // extra overhead produced by mask 'N' (archive number) characters.
-  // One 'N' character can result in several numbers if we process more
-  // than 9 archives.
-  wchar NewName[NM+MAX_GENERATE_MASK+20];
+  wchar NewName[NM];
 
   uint ArcNumber=1;
   while (true) // Loop for 'N' (archive number) processing.
@@ -833,7 +892,7 @@ void GenerateArchiveName(wchar *ArcName,size_t MaxSize,const wchar *GenerateMask
     
     bool ArcNumPresent=false;
 
-    GenArcName(NewName,GenerateMask,ArcNumber,ArcNumPresent);
+    GenArcName(NewName,ASIZE(NewName),GenerateMask,ArcNumber,ArcNumPresent);
     
     if (!ArcNumPresent)
       break;
@@ -845,7 +904,7 @@ void GenerateArchiveName(wchar *ArcName,size_t MaxSize,const wchar *GenerateMask
         // existing archive before the first unused name. So we generate
         // the name for (ArcNumber-1) below.
         wcsncpyz(NewName,NullToEmpty(ArcName),ASIZE(NewName));
-        GenArcName(NewName,GenerateMask,ArcNumber-1,ArcNumPresent);
+        GenArcName(NewName,ASIZE(NewName),GenerateMask,ArcNumber-1,ArcNumPresent);
       }
       break;
     }
@@ -895,8 +954,8 @@ bool GetWinLongPath(const wchar *Src,wchar *Dest,size_t MaxSize)
     {
       if (MaxSize<=PrefixLength+SrcLength)
         return false;
-      wcsncpy(Dest,Prefix,PrefixLength);
-      wcscpy(Dest+PrefixLength,Src);
+      wcsncpyz(Dest,Prefix,MaxSize);
+      wcsncatz(Dest,Src,MaxSize); // "\\?\D:\very long path".
       return true;
     }
     else
@@ -904,9 +963,9 @@ bool GetWinLongPath(const wchar *Src,wchar *Dest,size_t MaxSize)
       {
         if (MaxSize<=PrefixLength+SrcLength+2)
           return false;
-        wcsncpy(Dest,Prefix,PrefixLength);
-        wcscpy(Dest+PrefixLength,L"UNC");
-        wcscpy(Dest+PrefixLength+3,Src+1);
+        wcsncpyz(Dest,Prefix,MaxSize);
+        wcsncatz(Dest,L"UNC",MaxSize);
+        wcsncatz(Dest,Src+1,MaxSize); // "\\?\UNC\server\share".
         return true;
       }
     // We may be here only if we modify IsFullPath in the future.
@@ -923,9 +982,10 @@ bool GetWinLongPath(const wchar *Src,wchar *Dest,size_t MaxSize)
     {
       if (MaxSize<=PrefixLength+SrcLength+2)
         return false;
-      wcsncpy(Dest,Prefix,PrefixLength);
-      wcsncpy(Dest+PrefixLength,CurDir,2); // Copy drive letter 'd:'.
-      wcscpy(Dest+PrefixLength+2,Src);
+      wcsncpyz(Dest,Prefix,MaxSize);
+      CurDir[2]=0;
+      wcsncatz(Dest,CurDir,MaxSize); // Copy drive letter 'd:'.
+      wcsncatz(Dest,Src,MaxSize);
       return true;
     }
     else  // Paths in path\name format.
@@ -933,8 +993,8 @@ bool GetWinLongPath(const wchar *Src,wchar *Dest,size_t MaxSize)
       AddEndSlash(CurDir,ASIZE(CurDir));
       if (MaxSize<=PrefixLength+wcslen(CurDir)+SrcLength)
         return false;
-      wcsncpy(Dest,Prefix,PrefixLength);
-      wcscpy(Dest+PrefixLength,CurDir);
+      wcsncpyz(Dest,Prefix,MaxSize);
+      wcsncatz(Dest,CurDir,MaxSize);
 
       if (Src[0]=='.' && IsPathDiv(Src[1])) // Remove leading .\ in pathname.
         Src+=2;
@@ -960,16 +1020,17 @@ void ConvertToPrecomposed(wchar *Name,size_t NameSize)
 }
 
 
-// Remove trailing spaces and dots in file name and in dir names in path.
-void MakeNameCompatible(wchar *Name)
+void MakeNameCompatible(wchar *Name,size_t MaxSize)
 {
+  // Remove trailing spaces and dots in file name and in dir names in path.
   int Src=0,Dest=0;
   while (true)
   {
     if (IsPathDiv(Name[Src]) || Name[Src]==0)
       for (int I=Dest-1;I>0 && (Name[I]==' ' || Name[I]=='.');I--)
       {
-        if (IsPathDiv(Name[I-1])) // Permit path1/./path2 paths.
+        // Permit path1/./path2 and ../path1 paths.
+        if (Name[I]=='.' && (IsPathDiv(Name[I-1]) || Name[I-1]=='.' && I==1))
           break;
         Dest--;
       }
@@ -979,5 +1040,47 @@ void MakeNameCompatible(wchar *Name)
     Src++;
     Dest++;
   }
+
+  // Rename reserved device names, such as aux.txt to _aux.txt.
+  // We check them in path components too, where they are also prohibited.
+  for (uint I=0;Name[I]!=0;I++)
+    if (I==0 || I>0 && IsPathDiv(Name[I-1]))
+    {
+      static const wchar *Devices[]={L"CON",L"PRN",L"AUX",L"NUL",L"COM#",L"LPT#"};
+      wchar *s=Name+I;
+      bool MatchFound=false;
+      for (uint J=0;J<ASIZE(Devices);J++)
+        for (uint K=0;;K++)
+          if (Devices[J][K]=='#')
+          {
+            if (!IsDigit(s[K]))
+              break;
+          }
+          else
+            if (Devices[J][K]==0)
+            {
+              // Names like aux.txt are accessible without \\?\ prefix
+              // since Windows 11. Pure aux is still prohibited.
+              MatchFound=s[K]==0 || s[K]=='.' && !IsWindows11OrGreater() || IsPathDiv(s[K]);
+              break;
+            }
+            else
+              if (Devices[J][K]!=toupperw(s[K]))
+                break;
+      if (MatchFound)
+      {
+        wchar OrigName[NM];
+        wcsncpyz(OrigName,Name,ASIZE(OrigName));
+        if (MaxSize>I+1) // I+1, because we do not move the trailing 0.
+          memmove(s+1,s,(MaxSize-I-1)*sizeof(*s));
+        *s='_';
+#ifndef SFX_MODULE
+        uiMsg(UIMSG_CORRECTINGNAME,nullptr);
+        uiMsg(UIERROR_RENAMING,nullptr,OrigName,Name);
+#endif
+      }
+    }
 }
 #endif
+
+
