@@ -3,7 +3,7 @@
 int ToPercent(int64 N1,int64 N2)
 {
   if (N2<N1)
-    return(100);
+    return 100;
   return ToPercentUnlim(N1,N2);
 }
 
@@ -12,7 +12,7 @@ int ToPercent(int64 N1,int64 N2)
 int ToPercentUnlim(int64 N1,int64 N2)
 {
   if (N2==0)
-    return(0);
+    return 0;
   return (int)(N1*100/N2);
 }
 
