@@ -236,7 +236,7 @@ void RarTime::GetText(wchar *DateStr,size_t MaxSize,bool FullMS)
   else
   {
     // We use escape before '?' to avoid weird C trigraph characters.
-    wcscpy(DateStr,L"\?\?\?\?-\?\?-\?\? \?\?:\?\?");
+    wcsncpyz(DateStr,L"\?\?\?\?-\?\?-\?\? \?\?:\?\?",MaxSize);
   }
 }
 
@@ -271,14 +271,14 @@ void RarTime::SetIsoText(const wchar *TimeText)
 void RarTime::SetAgeText(const wchar *TimeText)
 {
   uint Seconds=0,Value=0;
-  for (int I=0;TimeText[I]!=0;I++)
+  for (uint I=0;TimeText[I]!=0;I++)
   {
-    int Ch=TimeText[I];
+    wchar Ch=TimeText[I];
     if (IsDigit(Ch))
       Value=Value*10+Ch-'0';
     else
     {
-      switch(etoupper(Ch))
+      switch(etoupperw(Ch))
       {
         case 'D':
           Seconds+=Value*24*3600;
