@@ -20,11 +20,13 @@ void ExtractStreams20(Archive &Arc,const wchar *FileName)
   wchar StreamName[NM+2];
   if (FileName[0]!=0 && FileName[1]==0)
   {
-    wcscpy(StreamName,L".\\");
-    wcscpy(StreamName+2,FileName);
+    // Convert single character names like f:stream to .\f:stream to
+    // resolve the ambiguity with drive letters.
+    wcsncpyz(StreamName,L".\\",ASIZE(StreamName));
+    wcsncatz(StreamName,FileName,ASIZE(StreamName));
   }
   else
-    wcscpy(StreamName,FileName);
+    wcsncpyz(StreamName,FileName,ASIZE(StreamName));
   if (wcslen(StreamName)+strlen(Arc.StreamHead.StreamName)>=ASIZE(StreamName) ||
       Arc.StreamHead.StreamName[0]!=':')
   {
@@ -35,7 +37,7 @@ void ExtractStreams20(Archive &Arc,const wchar *FileName)
 
   wchar StoredName[NM];
   CharToWide(Arc.StreamHead.StreamName,StoredName,ASIZE(StoredName));
-  ConvertPath(StoredName+1,StoredName+1);
+  ConvertPath(StoredName+1,StoredName+1,ASIZE(StoredName)-1);
 
   wcsncatz(StreamName,StoredName,ASIZE(StreamName));
 
@@ -83,15 +85,14 @@ void ExtractStreams(Archive &Arc,const wchar *FileName,bool TestMode)
   wchar FullName[NM+2];
   if (FileName[0]!=0 && FileName[1]==0)
   {
-    wcscpy(FullName,L".\\");
-    wcsncpyz(FullName+2,FileName,ASIZE(FullName)-2);
+    // Convert single character names like f:stream to .\f:stream to
+    // resolve the ambiguity with drive letters.
+    wcsncpyz(FullName,L".\\",ASIZE(FullName));
+    wcsncatz(FullName,FileName,ASIZE(FullName));
   }
   else
     wcsncpyz(FullName,FileName,ASIZE(FullName));
 
-  byte *Data=&Arc.SubHead.SubData[0];
-  size_t DataSize=Arc.SubHead.SubData.Size();
-
   wchar StreamName[NM];
   GetStreamNameNTFS(Arc,StreamName,ASIZE(StreamName));
   if (*StreamName!=':')
@@ -103,22 +104,30 @@ void ExtractStreams(Archive &Arc,const wchar *FileName,bool TestMode)
 
   if (TestMode)
   {
-    Arc.ReadSubData(NULL,NULL);
+    File CurFile;
+    Arc.ReadSubData(NULL,&CurFile,true);
     return;
   }
 
   wcsncatz(FullName,StreamName,ASIZE(FullName));
 
+
   FindData fd;
-  bool Found=FindFile::FastFind(FileName,&fd);
+  bool HostFound=FindFile::FastFind(FileName,&fd);
 
   if ((fd.FileAttr & FILE_ATTRIBUTE_READONLY)!=0)
     SetFileAttr(FileName,fd.FileAttr & ~FILE_ATTRIBUTE_READONLY);
   File CurFile;
-  if (CurFile.WCreate(FullName) && Arc.ReadSubData(NULL,&CurFile))
-    CurFile.Close();
+
+  if (CurFile.WCreate(FullName))
+  {
+    if (Arc.ReadSubData(NULL,&CurFile,false))
+      CurFile.Close();
+  }
+
+  // Restoring original file timestamps.
   File HostFile;
-  if (Found && HostFile.Open(FileName,FMF_OPENSHARED|FMF_UPDATE))
+  if (HostFound && HostFile.Open(FileName,FMF_OPENSHARED|FMF_UPDATE))
     SetFileTime(HostFile.GetHandle(),&fd.ftCreationTime,&fd.ftLastAccessTime,
                 &fd.ftLastWriteTime);
 
